// CDT31 by Philippe Chavatte - lormedy@free.fr - since 2018, july 25th
// Train controller written for DCCpp and S88 compatibility
// Use a PC serial link with "node.js" engine software, COMx ==> USB to Arduino MEGA with DCCpp software downloaded (@115200kbauds)
// Operates under control of 3 main files: CDT31.html, CDT31.js and CDT31server.js located in the same directory beside "node_modules" dir
// You can simultaneously control 99 locs and 512 accessories or turnouts on main and programming tracks
// Use 128 step to drive, speed counter is ajustable to your Loc maxi speed.
// Loc number use up to 5 numbers but only the last 4 are used as address.
// Display 2 x 256 sensors of the S88 retro-signalisation bus read thru DCCpp_S88 downloaded to Arduino MEGA
//
// Conduteur de train écrit pour être compatible avec DCCpp et S88 par Philippe Chavatte - lormedy@free.fr - depuis le 25 juillet 2018
// Utilise la liaison série du PC avec le moteur logiciel "node.js", COMx ==> USB vers un Arduino MEGA téléchargé avec le logiciel DCCpp (@115200kbaud)
// Fonctionne sous le controle de 3 fichiers principaux : CDT31.html, CDT31.js and CDT31server.js placés dans le répertoire qui contient aussi "node_modules"
// Vous pouvez controler simultanément 99 locomotives et 512 accessoires ou aiguillages, sur voie principale et de programmation
// Utilise 128 crans de traction, avec compteur de vitesse réglable à la vitesse maxi de votre loco,
// Numéro de loco sur 5 chiffres mais seulement les 4 derniers sont utilisés comme adresse.
// Affichage du bus de retro-signalisation S88 avec 2 x 256 capteurs lus par DCCpp_S88 téléchargé dans Arduino MEGA
//
// V1.0 commande DCCpp ou DCC++ - 10/09/2018
// V2.0 lecture et programmation des CV - 10/09/2018
// V2.1 lecture du courant sur voie principale - 15/12/2018
// V3.0 lecture du bus S88 compatible avec DCCpp_S88 - 15/04/2019
// V3.1 integration de websocket + json - 10/06/2019
//
// comm interface and websocket by Pierre N.
// software formating by Mathieu A.
// speedmeter and accessories models from Yaasan
// last update by Philippe Chavatte 2019, october 26th

var nbreAiguillage = 35;  // indiquez le nombre d'aiguillages, ensuite ce sera des accessoires 

var powerOnOff = 0;
var PowerStatus = 0;
// Locomotives
var LocAddr = 3;
var LocAddrProg = 127;
var LocSpeed = 0;
var LocDir = 1;
var um_Select = 1;
const maxStep = 126;
const maxLocAddr = 10239; // Limite DCC = 10240
const maxLocNbr = 99999;
const Speed_limit = 220;  // vitesse maxi = 220 km/h
var maxspeed = 160;
var LocFuncStatus = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];

var url = "";
var request = 0;
var sync = false;
var surchargeEnCourant = false;
var i2s;
var i2s_toggle = 0;

// Accessoires
var AccPageNo = 0;
var AccStatus = new Array(520, 0);

// Retro-signalisation S88
var S88PageNo = 0;
var S88Status = new Array(512, 0);
const binary = 0;
const hexa = 1;
const purehexa = 2;
const s88MaxByte = 64;    // les entrées sont comptées par groupe de 8 donc 64 * 8 = 512 bits
var s88Byte = 64;
var format_S88 = 0;       // toggle binAscii/nibbleAscii/ByteAscii/Qsensor

// Programmation des CV
const MAIN = 1;
const PROG = 0;
var trackSelect = PROG;   // PROG is default value
const maxCV = 1024;
var numCV = 0;
var CVRead = "999";
var CVvalue = 0;
var CVbitvalue = 0;
var CVbitselect = 0;

/* -------------------------- COMM ----------------------------------------- */

// Websocket
var wsClient = new WebSocket('ws://localhost:52560/');

wsClient.onopen = function () {
    console.log('websocket client connected ...');
    //    wsClient.send('CDT31 client connected');
}

wsClient.onmessage = function (ev) {
    var json = JSON.parse(ev.data);
    console.log(json);
    if (json.topic === 'data') {
        parseResponse(json.msg);
    }
}

// HTTP
function createXMLHttpRequest() {
    if (window.XMLHttpRequest) {
        return new XMLHttpRequest();
    }
    else if (window.ActiveXObject) {
        try {
            return new ActiveXObject("Msxml2.XMLHTTP");
        }
        catch (e) {
            try {
                return new ActiveXObject("Microsoft.XMLHTTP");
            }
            catch (e2) {
                return null;
            }
        }
    }
    else {
        return null;
    }
}

$(function () {
    request = createXMLHttpRequest();
    request2 = createXMLHttpRequest();
    request.onreadystatechange = (event) => {
        if (event.currentTarget.readyState === 4 && event.currentTarget.status !== 200) {
            $('#popup_message').css('visibility', 'visible');
            $('#popup_message').css('opacity', '1');
        }
    }

    request2.onreadystatechange = (event) => {
        if (event.currentTarget.readyState === 4 && event.currentTarget.status !== 200) {
            $('#popup_message').css('visibility', 'visible');
            $('#popup_message').css('opacity', '1');
        }
    };

    $("pop").click(function (event) {
        event.preventDefault();
        $('#popup_message').css('visibility', 'hidden');
        $('#popup_message').css('opacity', '0');
        $('#popup_displayConfig').css('visibility', 'hidden');
        $('#popup_displayConfig').css('opacity', '0');
    });
});

function send2Node(url, sync) {                 // send by XMLHttpRequest
    request = createXMLHttpRequest();
    if (!sync) get_Link_Answer();
    request.open('GET', url, sync);
    //  request.open("GET", url, false);        // lecture synchrone, attend une réponse donc bloquant !
    //  request.open("GET", url, true);         // lecture asynchrone, donc non bloquant !
    request.send(null);
}

// Lire la réponse de l'Arduino avec HttpRequest
function get_Link_Answer() {
    request.onreadystatechange = function (event) {
        if (this.readyState === 4 && (this.status === 200 || xhr.status === 0)) {
            parseResponse(this.responseText);
        }
        else {
            // showResponse("Erreur node");
            console.log('Status de la réponse : %d (%s) %d', this.status, this.statusText, this.readyState);
        }
    }
}

function parseResponse(msg) {
    var chn = msg;
    console.log("Réponse reçue: '%s'", chn);
    // showResponse(msg);
    // alert(msg); // Données textuelles récupérées
    if (chn.startsWith('<p')) {
        let inPower = parseInt(chn.replace('<p', '').replace('>', ''));
        console.log('power: ' + inPower);
        LocSpeed = 0;                  // vitesse à zéro quand on manoeuvre l'alimentation générale DCC
        switch (inPower) {
            case 0:
                //          document.getElementById("btnPower").innerText =("Power OFF");
                $("#btnPower").text("Power OFF");
                document.getElementById('tab-cab').style.backgroundColor = "#FF8080";
                document.getElementById('tab-prog').style.backgroundColor = "#FF8080";
                document.getElementById('cran2traction').value = LocSpeed;

                if (PowerStatus == 1) {
                    surchargeEnCourant = true;
                    chn += '     Surcharge en courant détectée !';
                    $("#slider").slider("value", LocSpeed);
                    PowerStatus = 0;
                }
                break;
            case 1:
                surchargeEnCourant = false;
                //          document.getElementById("btnPower").innerText =("Power ON");
                $("#btnPower").text("Power ON");
                document.getElementById('tab-cab').style.backgroundColor = "#E0FFE0";
                document.getElementById('tab-prog').style.backgroundColor = "#E0FFE0";
                break;
            case 2:
                chn += '     Surcharge en courant détectée sur voie principale !';
                break;
            case 3:
                chn += '     Surcharge en courant détectée sur voie de programmation !';
                break;
        }
        setVisibleItems(inPower & 1);
        onDrawMeter(40);
    }
    else if ((chn.startsWith('<r82|67|')) || (chn.startsWith('<r82|65|'))) { // Read CV
        CVRead = chn.split(" ")[1].replace(">", "");
        //        CVRead = chn;
        if (parseInt(CVRead) == -1) {
            chn += "     Erreur de lecture";
            CVRead = "------";
        }
        if ((chn.startsWith("<r82|67|19")) && (parseInt(CVRead) != 0)) {
            chn += "     UM ";
            chn += CVRead;
        }
        //        document.getElementById('spinnerCV').value = numCV.toString();
        document.getElementById('cvRead').value = CVRead;
    }
    else if ((chn.startsWith("<r87|67|")) || (chn.startsWith("<r87|98|"))) { // Write CV, Write bit
        CVRead = chn.split(" ")[1].replace(">", "");
        if (CVRead == -1) {
            chn += "     Erreur d'ecriture";
        }
    }
    else if (chn.startsWith("<y")) { // Read S88
        let s88chain = chn.split("y")[1].replace(">", "");
        let i = 0;
        let sensor = 0;

        switch (format_S88) {
            case binary:
                for (i = 0; i < (8 * s88Byte); i++) {
                    let sub_s88chain = (s88chain.substr((i) + 1, 1));
                    S88Status[i] = parseInt(sub_s88chain, 2);
                }
                for (i = i; i < (8 * s88MaxByte); i++) {
                    S88Status[i] = 2;
                }
                break;
            case hexa:
                for (i = 0; i < (2 * s88Byte); i++) {
                    let sub_s88chain = (s88chain.substr((i) + 1, 1));
                    sensor = parseInt(sub_s88chain, 16);
                    S88Status[4 * i] = (sensor & 0x08) == 0 ? 0 : 1;
                    S88Status[4 * i + 1] = (sensor & 0x04) == 0 ? 0 : 1;
                    S88Status[4 * i + 2] = (sensor & 0x02) == 0 ? 0 : 1;
                    S88Status[4 * i + 3] = (sensor & 0x01) == 0 ? 0 : 1;
                }
                for (i = 4 * i; i < (8 * s88MaxByte); i++) {
                    S88Status[i] = 2;
                }
                break;
            case purehexa:
                for (i = 0; i < s88Byte; i++) {
                    let sub_s88chain = (s88chain.substr((i) + 1, 1));
                    sensor = parseInt(sub_s88chain, 16);
                    S88Status[8 * i] = (sensor & 0x80) == 0 ? 0 : 1;
                    S88Status[8 * i + 1] = (sensor & 0x40) == 0 ? 0 : 1;
                    S88Status[8 * i + 2] = (sensor & 0x20) == 0 ? 0 : 1;
                    S88Status[8 * i + 3] = (sensor & 0x10) == 0 ? 0 : 1;
                    S88Status[8 * i + 4] = (sensor & 0x08) == 0 ? 0 : 1;
                    S88Status[8 * i + 5] = (sensor & 0x04) == 0 ? 0 : 1;
                    S88Status[8 * i + 6] = (sensor & 0x02) == 0 ? 0 : 1;
                    S88Status[8 * i + 7] = (sensor & 0x01) == 0 ? 0 : 1;
                }
                for (i = 8 * i; i < (8 * s88MaxByte); i++) {
                    S88Status[i] = 2;
                }
                break;
            default:
                break;
        }
        drawS88Panel();
    }
    else if (chn.startsWith("<a")) { // Read current
        let current = chn.split("a")[1].replace(">", "");
        let c_temp = parseInt(current, 10);
        c_temp = (c_temp == 0 ? 0 : 10.1 * c_temp + 70);    // valeur approchée
        c_temp = Math.round(c_temp);
        current = c_temp.toString();
        chn = " Voie principale    ===>    " + current + " mA";
    }
    else if (chn.startsWith("{")) { // Do not display JSON
        return;
    }
    else if (!(chn.startsWith("<"))) { // Display Arduino config
        $('#popup_displayConfig').css('visibility', 'visible');
        $('#popup_displayConfig').css('opacity', '1');
        if (surchargeEnCourant == true) chn += 'Surcharge de courant détectée sur voies principales !';
        document.getElementById('displayConfig').innerText += chn;
    }

    showResponse(chn);
}

// Quand les paquets ont étés envoyés, fonction qui permet l'affichage du callback...
showResponse = function (response) {
    document.getElementById('log').value = response;
}

/* -------------------------- DCC functions -------------------------------- */

function onClickPower(outPower) {
    powerOnOff = outPower;
    url = "./?CMD=sendCommand(" + outPower + ")";
    send2Node(url, !sync);
    LocSpeed = 0;                               // vitesse à zéro quand on manoeuvre l'alimentation générale DCC
    $("#slider").slider("value", LocSpeed);
    setVisibleItems(outPower);
    onDrawMeter(40);
    if (outPower == 0) {
        document.getElementById('tab-cab').style.backgroundColor = "#FF8080";
        document.getElementById('tab-prog').style.backgroundColor = "#FF8080";
    }
    else {
        document.getElementById('tab-cab').style.backgroundColor = "#E0FFE0";
        document.getElementById('tab-prog').style.backgroundColor = "#E0FFE0";
    }
}

function onClickDisplayConfig() {
    url = "./?CMD=sendCommand(" + "C" + ")";
    send2Node(url, !sync);
    //    wsClient.send("./?CMD=sendCommand(" + "C" + ")");
}

function onClickPlus(inVal) {
    $("#spinner").spinner("value", parseInt(LocAddr) + inVal);
    //  $("#LocAddr_label").text(LocAddr);
}

function onClickPlusV(inSpeedStep) {            // incr./décr. la vitesse maxi du cadran
    $("#spinnerV").spinner("value", parseInt(maxspeed) + inSpeedStep);
    $("#maxspeed_label").text(maxspeed);
}

function onClickDir(inDir) {
    if (LocDir != inDir) {
        LocDir = inDir;
        if (LocDir == 1) {                      // Changer la couleur du fond AV/AR
            document.getElementById('btnFwd').style.backgroundColor = '#FFFFC0';
            document.getElementById('btnRev').style.backgroundColor = '';
        }
        else {
            document.getElementById('btnFwd').style.backgroundColor = '';
            document.getElementById('btnRev').style.backgroundColor = '#FFFFC0';
        }

        LocSpeed = 0;
        if (PowerStatus == 1) {                 // Rafraichir les indicateurs de vitesse
            $("#slider").slider("value", LocSpeed);
            // onChangeSpeed(LocSpeed); // done above
        }
    }
}

function onClickStop() {
    LocSpeed = 0;
    let aLocSpeed = -1;
    onChangeSpeed(aLocSpeed);
    $("#slider").slider("value", LocSpeed);
}

function onChangeSpeed(inSpeed) {
    inSpeed = (inSpeed > 126) ? 126 : inSpeed;
    LocSpeed = (inSpeed < 0) ? 0 : inSpeed;     // mémorise la vitesse de la loco
    // $("#slider").slider("value", LocSpeed);  // done, must stay in comment
    document.getElementById('cran2traction').value = LocSpeed;
    //  $("#cran2traction").val(LocSpeed);
    onDrawMeter(40);

    let aLocAddr = LocAddr;
    while (aLocAddr > maxLocAddr) {             // Limite DCC= 10240
        aLocAddr -= 10000;
    }
    url = "./?CMD=setLocoSpeedDir(" + aLocAddr + "," + inSpeed + "," + LocDir + ")";
    send2Node(url, !sync);
    //    wsClient.send("./?CMD=setLocoSpeedDir(" + aLocAddr + "," + inSpeed + "," + LocDir + ")");
}

function onClickFunction(inFuncNo) {
    if (PowerStatus == 0) {
        return;
    }
    let aOnOff = LocFuncStatus[inFuncNo];
    aOnOff = (aOnOff == 1) ? 0 : 1;
    LocFuncStatus[inFuncNo] = aOnOff;

    let aLocAddr = LocAddr;
    while (aLocAddr > maxLocAddr) {             // Limite DCC = 10240
        aLocAddr -= 10000;
    }

    url = "./?CMD=setLocoFunction(" + aLocAddr + "," + inFuncNo + "," + aOnOff + ")";
    send2Node(url, !sync);
    //    wsClient.send("./?CMD=setLocoFunction(" + aLocAddr + "," + inFuncNo + "," + aOnOff + ")");
}

function ChangeAcc(inNo) {
    let aOnOff = AccStatus[inNo];
    aOnOff = (1 - aOnOff) & 1;
    AccStatus[inNo] = aOnOff;
    let aAccAddr = inNo;

    url = "./?CMD=setTurnout(" + aAccAddr + "," + aOnOff + ")";
    send2Node(url, !sync);
    //    wsClient.send("./?CMD=setTurnout(" + aAccAddr + "," + aOnOff + ")");
}

function onClickProgTrackSelect(track) {        // Selectionne PROG / MAIN
    trackSelect = track;
    if (trackSelect == PROG) {                  // PROG
        document.getElementById('btnProgSelect').style.backgroundColor = '#FFFFC0';
        document.getElementById('btnMainSelect').style.backgroundColor = '';
        if (PowerStatus == 1) {
            $("#btnLocAddrRead").removeAttr('disabled');
            $("#btnLocAddrWrite").removeAttr('disabled');
            $("#btnProgRead").removeAttr('disabled');
        }
        $("#btnLocAddrRead").css('text-decoration', '');
        $("#btnLocAddrWrite").css('text-decoration', '');
        $("#btnProgRead").css('text-decoration', '');
    }
    else {                                      // MAIN
        document.getElementById('btnProgSelect').style.backgroundColor = '';
        document.getElementById('btnMainSelect').style.backgroundColor = '#FFFFC0';
        $("#btnLocAddrRead").attr('disabled', true);
        $("#btnLocAddrRead").css('text-decoration', 'line-through double');
        $("#btnLocAddrWrite").attr('disabled', true);
        $("#btnLocAddrWrite").css('text-decoration', 'line-through double');
        $("#btnProgRead").attr('disabled', true);
        $("#btnProgRead").css('text-decoration', 'line-through double');
    }
}

function onClickPlusLoco(inaddressVal) {        // incr./décr. l'adresse de la loco à programmer
    $("#spinnerLocProg").spinner("value", parseInt(LocAddrProg) + inaddressVal);
}

function onClickPlusCV(inCVnumVal) {            // incr./décr. le numéro du CV à lire/programmer
    $("#spinnerCV").spinner("value", numCV + inCVnumVal);
}

function onClickToggleUM() {
    (um_Select = 20 - um_Select);
    document.getElementById('UM_Select').value = (document.getElementById('UM_Select').value == false) ? true : false;
}

function onSelectReadLocAddr() {                // Lit l'adresse de la loco sur la voie de programmation, PROG only
    if ((PowerStatus == 0) || (trackSelect == MAIN)) {
        return;
    }

    $("#btnLocAddrRead").attr('disabled', true);
    //    wsClient.send("./?CMD=getLocAddr(" + trackSelect + ")");
    document.getElementById('spinnerLocProg').value = "";
    //    $("#spinnerLocProg").val("");

    numCV = "29";                               /// lire CV#29, si bit_5 == 1 lire CV#17 et CV#18 sinon lire CV#1
    onSelectReadCV();
    $("#spinnerCV").val(numCV);
    let aresponse = parseInt(CVRead);

    if (aresponse === -1) {
        $("#btnLocAddrRead").removeAttr('disabled');
        return;
    }
    else {
        if ((aresponse & 0x20) === 32) {
            // adresse longue : lire CV#17 et CV#18
            numCV = "18";
            onSelectReadCV();
            $("#spinnerCV").val(numCV);
            aresponse = parseInt(CVRead);

            if (aresponse === -1) {
                $("#btnLocAddrRead").removeAttr('disabled');
                return;
            }
            else {
                numCV = "17";
                onSelectReadCV();
                $("#spinnerCV").val(numCV);
                var bresponse = parseInt(CVRead);

                if (bresponse === -1) {
                    $("#btnLocAddrRead").removeAttr('disabled');
                    return;
                }
                else {
                    aresponse += (bresponse & 0x3F) << 8;
                }
            }
        }
        else {
            // adresse courte : lire CV#1
            numCV = "1";
            onSelectReadCV();
            $("#spinnerCV").val(numCV);
            aresponse = parseInt(CVRead);
        }
    }

    // adresse UM : lire CV#19 pour voir si elle est active !
    numCV = 19;
    onSelectReadCV();
    $("#spinnerCV").val(numCV);
    document.getElementById('spinnerLocProg').value = aresponse;
    // $("#spinnerLocProg").val(aresponse);
    $("#btnLocAddrRead").removeAttr('disabled');
}

function onSelectReadCV() {                     // Lit le CV de la loco sur la voie de programmation, PROG only
    if ((PowerStatus == 0) || (trackSelect == MAIN)) {
        console.log('err : PowerStatus == 0 ....');
        return;
    }

    $("#onSelectReadCV").attr('disabled', true);
    document.getElementById('cvWrite').value = "";

    let aLocAddr = LocAddrProg;
    url = "./?CMD=getLocoConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + ")";
    send2Node(url, sync);
    //    wsClient.send("./?CMD=getLocoConfig(" + numCV + ")");
    if (numCV == 1) document.getElementById('spinnerLocProg').value = CVRead;
    $("#onSelectReadCV").removeAttr('disabled');
}

function onSelectWriteLocAddr() {               // Ecrit l'adresse de la loco sur la voie de programmation, PROG only
    if ((PowerStatus == 0) || (trackSelect == MAIN)) {
        return;
    }

    let aLocAddr = LocAddrProg;
    $("#btnLocAddrWrite").attr('disabled', true);

    var LocAddress = parseInt(document.getElementById('spinnerLocProg').value);
    //  var LocAddress = parseInt($("#spinnerLocProg").val());
    if (LocAddress < 128) {
        // adresse courte : ecrire CV#1 ou CV#19
        var cv29_bit5 = 0;
        numCV = um_Select;
        //      numCV = ($("#UM_Select").val == "false") ? 1 : 19;
    }
    else {
        // adresse longue : ecrire CV#17 et CV#18
        cv29_bit5 = 1;
        numCV = 17;

        document.getElementById('spinnerCV').value = numCV;
        //      $("#spinnerCV").val(numCV);
        CVvalue = (LocAddress >> 8) | 0xC0;
        document.getElementById('cvWrite').value = CVvalue;
        //      $("#cvWrite").val(CVvalue);

        url = "./?CMD=setLocoConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVvalue + ")";
        send2Node(url, sync);
        //        wsClient.send("./?CMD=setLocoConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVvalue + ")");
        numCV = 18;
    }

    document.getElementById('spinnerCV').value = numCV;
    //  $("#spinnerCV").val(numCV);
    CVvalue = LocAddress & 0xFF;
    document.getElementById('cvWrite').value = CVvalue;
    //  $("#cvWrite").val(CVvalue);

    url = "./?CMD=setLocoConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVvalue + ")";
    send2Node(url, sync);
    //    wsClient.send("./?CMD=setLocoConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVvalue + ")");

    if (numCV != 19) {
        numCV = 29;                             // ecrire CV#29, bit_5 = 0/1
        CVbitselect = 5;
        CVbitvalue = cv29_bit5 & 1;
        document.getElementById('spinnerCV').value = numCV;
        document.getElementById('cv_Bit_Select').value = CVbitselect;
        document.getElementById('cv_Bit_Write').value = cv29_bit5;
        //      $("#spinnerCV").val(numCV);
        //      $("#cv_Bit_Select").val(CVbitselect);
        //      $("#cv_Bit_Write").val(cv29_bit5);

        url = "./?CMD=setLocoBitConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVbitselect + "," + CVbitvalue + ")";
        send2Node(url, sync);
        //        wsClient.send("./?CMD=setLocoBitConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVbitselect + "," + CVbitvalue + ")");
    }
    $("#btnLocAddrWrite").removeAttr('disabled');
}

function onSelectWriteCV() {                    // Ecriture d'un CV ==> tracks : Main + PROG
    if (PowerStatus == 0) {
        console.log('err :PowerStatus == 0 .....');
        return;
    }

    document.getElementById('cvRead').value = "";

    let aLocAddr = LocAddrProg;
    // lire la valeur du CV à écrire
    CVvalue = document.getElementById('cvWrite').value;
    //  CVvalue = $("#cvWrite").val();

    url = "./?CMD=setLocoConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVvalue + ")";
    //console.log('command WriteCV to DCCpp ...', url);
    send2Node(url, sync);
    //    wsClient.send("./?CMD=setLocoConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVvalue + ")");
}

function onSelectWriteCVBit() {                 // Ecriture d'un bit dans le CV ==> tracks : Main + PROG
    if (PowerStatus == 0) {
        return;
    }

    document.getElementById('cvRead').value = "";
    document.getElementById('cvWrite').value = "";

    let aLocAddr = LocAddrProg;
    // lire le numéro du bit CV à écrire
    let CVbitselect = document.getElementById('cv_Bit_Select').value & 7;
    //  let CVbitselect = $("#cv_Bit_Select").val() & 7;
    // lire la valeur du bit CV à écrire
    CVbitvalue = document.getElementById('cv_Bit_Write').value & 1;
    //  CVbitvalue = $("#cv_Bit_Write").val() & 1;

    url = "./?CMD=setLocoBitConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVbitselect + "," + CVbitvalue + ")";
    send2Node(url, sync);
    //    wsClient.send("./?CMD=setLocoBitConfig(" + aLocAddr + "," + trackSelect + "," + numCV + "," + CVbitselect + "," + CVbitvalue + ")");
}

function read_Current_Main() {                  // Lecture du courant DCC sur voies principales
    if (PowerStatus == 0) {
        return;
    }

    url = "./?CMD=sendCommand(" + "c" + ")";
    send2Node(url, sync);
    //    wsClient.send("./?CMD=sendCommand(" + "c" + ")");
}

// lire le bus S88, 64 modules de 8 capteurs maxi
function onClickRead_S88() {
    if (format_S88 == 3) {
        url = "./?CMD=sendCommand(" + "Q" + "," + s88Byte + "," + format_S88 + ")"; // 3 Qsensor style
    }
    else {
        url = "./?CMD=sendCommand(" + "Y" + "," + s88Byte + "," + format_S88 + ")"; // 0 binAscii, 1 nibbleAscii, 2 ByteAscii
    }
    send2Node(url, sync);
    //    wsClient.send(url);

    if (s88Byte == 0) {
        for (var i = 0; i < 512; i++) {
            S88Status[i] = 2;
        }

        drawS88Panel();
    }
}

function toggle_S88_Mode() {
    format_S88 = (format_S88 + 1) % 4;
    document.getElementById('btnS88Protocol').innerText = ((format_S88 == 0 ? "S88 b" : (format_S88 == 1 ? "S88 h" : (format_S88 == 2 ? "S88 x" : "Q / q"))));
    //  $("#btnS88Protocol").text((format_S88 == 0 ? "S88 b" : (format_S88 == 1 ? "S88 h" : (format_S88 == 2 ? "S88 x" : "S88 Q"))));
}

/* -------------------------- Buttons -------------------------------------- */

$(function () {
    $("#slider").slider({
        animate: "fast",
        min: 0,
        max: maxStep,
        range: false,
        step: 1,  // default: 1
        value: 0, // default: 0
        orientation: 'horizontal', // 'horizontal' or 'vertical'
        change: function (event, ui) {
            if ((PowerStatus == 1) || (LocSpeed == 0)){
                //      $(ui.button.text).find('span').html(cran2traction);
                onChangeSpeed(ui.value);
            }
        },
        slide: function (event, ui) {
            if ((PowerStatus == 1) || (LocSpeed == 0)){
                onChangeSpeed(ui.value);
            }
        }
    });

    $("#slider").css('background', '#00C0FF');   // fond de la piste
    $("#slider").css('border-color', '#FF2929'); // contour de la piste

    $("input[type=submit], a, button")
        .button()
        .click(function (event) {
            event.preventDefault();
        });
});

$(function () {
    var spinner = $("#spinner").spinner({       // Adresse loco voie principale
        min: 0,
        max: maxLocNbr,
        step: 1,

        change: function (event, ui) {
            LocAddr = spinner.spinner("value");
        }
    });

    var spinnerV = $("#spinnerV").spinner({     // Vitesse loco cadran
        min: 60,
        max: Speed_limit,
        step: 10,

        change: function (event, ui) {
            _speed = spinnerV.spinner("value");

            if (_speed > Speed_limit) {
                maxspeed = Speed_limit;
            }
            else if (_speed < 60) {
                maxspeed = 60;
            }
            else {
                maxspeed = _speed;
            }

            console.log('spinnerV: ' + maxspeed);
            onDrawMeter(40);
        }
    });

    var spinnerLocProg = $("#spinnerLocProg").spinner({ // Adresse loco à programmer
        min: 0,
        max: maxLocNbr,
        change: function (event, ui) {
            LocAddrProg = spinnerLocProg.spinner("value");
        }
    });

    var spinnerCV = $("#spinnerCV").spinner({   // Numéro de CV
        min: 1,
        max: maxCV,
        change: function (event, ui) {
            numCV = spinnerCV.spinner("value");
        }
    });

    var spinnerS88 = $("#spinnerS88").spinner({ // Nombre d'octets du S88
        min: 0,
        step: 2,
        max: s88MaxByte,
        change: function (event, ui) {
            var temp = spinnerS88.spinner("value");
            if (temp > s88MaxByte) {
                s88Byte = s88MaxByte;
                document.getElementById("spinnerS88").value = s88MaxByte;
            }
            else
                s88Byte = temp;
        }
    });

    $("button").button();

});

$(function () {
    setVisibleItems(0);

    $("#check0").button();
    $("#check1").button();
    $("#check2").button();
    $("#check3").button();
    $("#check4").button();
    $("#check5").button();
    $("#check6").button();
    $("#check7").button();
    $("#check8").button();
    $("#check9").button();
    $("#check10").button();
    $("#check11").button();
    $("#check12").button();
    $("#check13").button();
    $("#check14").button();
    $("#check15").button();
    $("#check16").button();
    $("#check17").button();
    $("#check18").button();
    $("#check19").button();
    $("#check20").button();
    $("#check21").button();
    $("#check22").button();
    $("#check23").button();
    $("#check24").button();
    $("#check25").button();
    $("#check26").button();
    $("#check27").button();
    $("#check28").button();
    $("#checkDCC").button();
    $("#displayStatus").button();
    $('#tabControl').tabs({ fx: { opacity: 'toggle', duration: 'fast' } });

    for (var i = 0; i < 512; i++) {
        AccStatus[i] = 0;
        S88Status[i] = 2;
    }
});

function setVisibleItems(inOnSwitch) {
    if (inOnSwitch == 0) {
        PowerStatus = 0;
        document.getElementById('signal').width = '52';
        document.getElementById('signal').height = '52';
        document.getElementById('signal').src = './public/img/CarreMecanique.png';
        document.getElementById('signal').setAttribute('title', 'Arrêt absolu');
        $("#powerOff").hide('normal');
        $("#powerOn").show('normal');
        $("#btnStop").attr('disabled', true);
        $("#btnRev").attr('disabled', true);
        $("#btnFwd").attr('disabled', true);
        $("#btnRead_Current").attr('disabled', true);
        //  $("#btnProtocol").attr('disabled', true);
        $("#btnProgProtocol").attr('disabled', true);
        $("#slider").attr('disabled', true);
        $("#btnLocAddrRead").attr('disabled', true);
        $("#btnLocAddrWrite").attr('disabled', true);
        $("#btnProgRead").attr('disabled', true);
        $("#btnProgWrite").attr('disabled', true);
        $("#btnProgBitWrite").attr('disabled', true);
        $("#btnAccProtocol").button().attr('disabled', true);
        $("#btnDCCProtocol").button().attr('disabled', true);
        $("#btnS88Protocol").removeAttr('disabled')
        $("#check0").button().attr('disabled', true);
        $("#check1").button().attr('disabled', true);
        $("#check2").button().attr('disabled', true);
        $("#check3").button().attr('disabled', true);
        $("#check4").button().attr('disabled', true);
        $("#check5").button().attr('disabled', true);
        $("#check6").button().attr('disabled', true);
        $("#check7").button().attr('disabled', true);
        $("#check8").button().attr('disabled', true);
        $("#check9").button().attr('disabled', true);
        $("#check10").button().attr('disabled', true);
        $("#check11").button().attr('disabled', true);
        $("#check12").button().attr('disabled', true);
        $("#check13").button().attr('disabled', true);
        $("#check14").button().attr('disabled', true);
        $("#check15").button().attr('disabled', true);
        $("#check16").button().attr('disabled', true);
        $("#check17").button().attr('disabled', true);
        $("#check18").button().attr('disabled', true);
        $("#check19").button().attr('disabled', true);
        $("#check20").button().attr('disabled', true);
        $("#check21").button().attr('disabled', true);
        $("#check22").button().attr('disabled', true);
        $("#check23").button().attr('disabled', true);
        $("#check24").button().attr('disabled', true);
        $("#check25").button().attr('disabled', true);
        $("#check26").button().attr('disabled', true);
        $("#check27").button().attr('disabled', true);
        $("#check28").button().attr('disabled', true);
    }
    else {
        PowerStatus = 1;
        document.getElementById('signal').width = '52';
        document.getElementById('signal').height = '77';
        document.getElementById('signal').src = './public/img/vl.png';
        document.getElementById('signal').setAttribute('title', 'Voie libre');
        $("#powerOn").hide('normal');
        $("#powerOff").show('normal');
        $("#btnStop").removeAttr("disabled");
        $("#btnRev").removeAttr("disabled");
        $("#btnFwd").removeAttr("disabled");
        $("#btnRead_Current").removeAttr("disabled");
        //  $("#btnProtocol").attr('disabled', true);
        $("#btnProgProtocol").removeAttr("disabled");
        $("#slider").removeAttr("disabled");
        if (trackSelect == PROG) $("#btnLocAddrRead").removeAttr('disabled');
        if (trackSelect == PROG) $("#btnLocAddrWrite").removeAttr('disabled')
        if (trackSelect == PROG) $("#btnProgRead").removeAttr('disabled');
        $("#btnProgWrite").removeAttr('disabled');
        $("#btnProgBitWrite").removeAttr('disabled');
        $("#btnAccProtocol").button().attr('disabled', true);
        $("#btnDCCProtocol").button().attr('disabled', true);
        $("#btnS88Protocol").removeAttr('disabled')
        $("#check0").button().removeAttr('disabled');
        $("#check1").button().removeAttr('disabled');
        $("#check2").button().removeAttr('disabled');
        $("#check3").button().removeAttr('disabled');
        $("#check4").button().removeAttr('disabled');
        $("#check5").button().removeAttr('disabled');
        $("#check6").button().removeAttr('disabled');
        $("#check7").button().removeAttr('disabled');
        $("#check8").button().removeAttr('disabled');
        $("#check9").button().removeAttr('disabled');
        $("#check10").button().removeAttr('disabled');
        $("#check11").button().removeAttr('disabled');
        $("#check12").button().removeAttr('disabled');
        $("#check13").button().removeAttr('disabled');
        $("#check14").button().removeAttr('disabled');
        $("#check15").button().removeAttr('disabled');
        $("#check16").button().removeAttr('disabled');
        $("#check17").button().removeAttr('disabled');
        $("#check18").button().removeAttr('disabled');
        $("#check19").button().removeAttr('disabled');
        $("#check20").button().removeAttr('disabled');
        $("#check21").button().removeAttr('disabled');
        $("#check22").button().removeAttr('disabled');
        $("#check23").button().removeAttr('disabled');
        $("#check24").button().removeAttr('disabled');
        $("#check25").button().removeAttr('disabled');
        $("#check26").button().removeAttr('disabled');
        $("#check27").button().removeAttr('disabled');
        $("#check28").button().removeAttr('disabled');
    }
}

/* -------------------------- Drawing -------------------------------------- */

var CenterX = 0;
var CenterY = 0;

function onLoadInit() {                         // Initialisation
    onDraw();
    onClickProgTrackSelect(PROG);
    document.getElementById('btnRead_Current').innerHTML = "DCC";
    document.getElementById('cran2traction').value = LocSpeed;
    //  $("#cran2traction").val(LocSpeed);
    if (LocDir == 1) {
        $("#btnFwd").css('background-color', '#FFFFC0');
        $("#btnRev").css('background-color', '');
    }
    else {
        $("#btnFwd").css('background-color', '');
        $("#btnRev").css('background-color', '#FFFFC0');
    }
}

function drawLoco() {                           // ajoute une image dans le cadran
    let ctx = document.getElementById("myCanvas").getContext("2d");
    let img = new Image();
    img.src = (LocDir == 1) ? "/public/img/BB25245F.jpg" : "/public/img/BB25245R.jpg";
    img.onload = function () {
        ctx.drawImage(img, 125, 315);
    }
}

function onDraw() {
    onDrawMeter(40);
    drawAccPanel();
    drawS88Panel();
}

// Canvas 描画処理 Processus de dessin du compteur de vitesse
function onDrawMeter(inScale) {
    drawMeter(LocSpeed, inScale);
}

// Compteur de vitesse

function drawMeter(inSpeed, inScale) {

    var inValue = Math.round((inSpeed * 270) / maxStep);
    var canvas = document.getElementById("myCanvas");
    var cv = canvas.getContext("2d");
    var scale = inScale * 10
    var center = { x: scale / 2, y: scale / 2 };
    var hLen = scale * 0.8 / 2; // 針の長さ Longueur d'aiguille
    var fSize = scale / 9;  // フォントサイズ  Taille de police
    var rLen = scale / 2;

    CenterX = center.x;
    CenterY = center.y;

    canvas.addEventListener("mousedown", onClickCanvas);
    //canvas.addEventListener("mousemove", onClickCanvas);

    //cv.save();
    // メーター背景の円描画 Dessin de cercle du fond de vumètre
    cvSize = document.getElementById("myCanvas").getAttribute("width");
    cv.clearRect(0, 0, cvSize, cvSize);

    //背景を白に描画 Dessinez le fond en couleur
    cv.fillStyle = ((PowerStatus == 0) ? "#FF8080" : "#E0FFE0");
    cv.fillRect(0, 0, cvSize, cvSize);

    //Draw meter
    cv.beginPath();
    cv.arc(center.x, center.y, scale / 2, 0, Math.PI * 2, false);
    // グラデーション指定 Spécification de dégradé
    var grad = cv.createRadialGradient(center.x / 1, center.y / 1, 0, center.x, center.y, scale / 2);
    grad.addColorStop(0.00, "#202030");
    grad.addColorStop(0.92, "#404050");
    grad.addColorStop(0.95, "#0A0A0A");
    grad.addColorStop(0.97, "#707070");
    //          grad.addColorStop(1.00, "#8A8A8A");
    grad.addColorStop(1.00, "#00C0FF"); // bleu
    cv.fillStyle = grad;
    cv.fill();

    // Canvas - Couleur des points
    //      cv.fillStyle    = "#9F9F9F";
    cv.fillStyle = "#80FFFF"; // couleur des points du cadran vitesse

    var counterstep = maxspeed / (((((maxspeed >> 1) & 1) == 0) || (maxspeed >= 140)) ? 20 : 15);
    if (maxspeed == 150) counterstep = parseInt(maxspeed / 15);
    if (maxspeed < 131) counterstep = parseInt(maxspeed / 10);
    if (maxspeed == 105) counterstep = 7;
    if (maxspeed == 75) counterstep -= 2;
    var nbpoints = counterstep * 5;

    // メモリ表示 Affichage des points
    //      for(var i=0;i<=maxspeed/4;i++){
    //          var radian = ((270 / (maxspeed/4)) * i + 135) * Math.PI / 180;
    for (var i = 0; i <= nbpoints; i++) {
        var radian = ((270 / nbpoints) * i + 135) * Math.PI / 180;
        var beginLineOffset = 10;
        var endLineOffset = 3;

        if (i % 5 == 0) {
            beginLineOffset = 2;
            cv.lineWidth = scale / 100;
        }
        else {
            //beginLineOffset = 10;
            cv.lineWidth = scale / 133;
        }

        var x_begin = center.x + (rLen - 40 + beginLineOffset) * Math.cos(radian);
        var y_begin = center.y + (rLen - 40 + beginLineOffset) * Math.sin(radian);
        var x_end = center.x + (rLen - 13 - endLineOffset) * Math.cos(radian);
        var y_end = center.y + (rLen - 13 - endLineOffset) * Math.sin(radian);

        cv.strokeStyle = "#FFFFFF"; // couleur des traits du cadran vitesse
        cv.beginPath();
        cv.moveTo(x_begin, y_begin);
        cv.lineTo(x_end, y_end);
        cv.closePath();
        cv.stroke();

        cv.fill();
    }

    // Canvas の色、フォント Couleur de police
    //      cv.fillStyle    = "#FFFFFF";
    cv.fillStyle = "#80FFFF"; // couleur des chiffres du cadran vitesse
    cv.font = fSize * 0.55 + "px 'arial'";
    cv.shadowColor = '#9F9F9F';
    cv.shadowOffsetX = 0;
    cv.shadowOffsetY = 0;
    cv.shadowBlur = 16;
    // 文字盤表示 Afficheur de cadran
    //      for(var i=0;i<=20;i++){
    for (let i = 0; i <= counterstep; i++) {
        //          var radian = ((270 / 20) * i + 135) * Math.PI / 180;
        var radian = ((270 / counterstep) * i + 135) * Math.PI / 180;
        var xx = center.x + (hLen - 25) * Math.cos(radian) - fSize / 4;
        var yy = center.y + (hLen - 25) * Math.sin(radian) + fSize / 4;
        let aMetrics3 = cv.measureText("0");
        //          cv.fillText(i * 30, xx - (aMetrics3.width / 2), yy);
        //          cv.fillText(i * maxspeed/20, xx - (aMetrics3.width / 2), yy); // dessine un compteur de 0 à "maxspeed" km/h
        let value = parseInt(i * maxspeed / counterstep);
        cv.fillText(value, xx - (aMetrics3.width / 2), yy); // dessine un compteur de 0 à "maxspeed" km/h
    }

    // 値表示 Affichage de la valeur
    cv.fillStyle = "#FFFFFF";
    cv.font = "bold " + fSize * 0.7 + "px 'verdana'";
    // nom du projet  + version
    cv.fillText("CDT31", center.x - 58, center.y - scale / 6.5);

    var aMetrics = cv.measureText(Math.round(inSpeed * maxspeed / maxStep)); // calcule la vitesse de 0 à "maxspeed" km/h
    cv.fillText(Math.round(inSpeed * maxspeed / maxStep), center.x - (aMetrics.width / 2), center.y + scale / 5); // affiche la vitesse de 0 à "maxspeed" km/h
    cv.font = "bold " + fSize * 0.5 + "px 'verdana'";
    var aMetrics2 = cv.measureText("km/h");
    cv.fillText("km/h", center.x - (aMetrics2.width / 2), center.y + scale / 3.8);

    cv.font = "bold " + fSize * 0.4 + "px 'verdana'";
    cv.fillStyle = "#0000FF"; // bleu
    if (LocDir == 1) {
        cv.fillText("AV >>", center.x + 120, center.y + scale / 2.15);
    }
    else {
        cv.fillText("<< AR", center.x - 200, center.y + scale / 2.15);
    }

    drawLoco(); // ici en bas du cadran

    cv.shadowBlur = 0;
    cv.shadowColor = "#444";
    cv.shadowOffsetX = 0;
    cv.shadowOffsetY = 0;

    // 針(影)描画 Dessin d'aiguille (ombre)
    let hRadian = (inValue + 135) * Math.PI / 180;
    // var pos_x = cPoint(center, rLen - 25, hRadian);
    let pos_x = cPoint(center, rLen - 40, hRadian);
    let a90deg = Math.PI / 2;
    cv.strokeStyle = "#550000";
    cv.lineWidth = scale / 50;
    cv.beginPath();
    cv.moveTo(center.x + 8 * Math.cos(hRadian - a90deg), center.y + 8 * Math.sin(hRadian - a90deg));
    cv.lineTo(pos_x.x, pos_x.y);
    cv.lineTo(center.x + 8 * Math.cos(hRadian + a90deg), center.y + 8 * Math.sin(hRadian + a90deg));
    cv.closePath();
    cv.stroke();

    // 針(中心)描画 Dessin aiguille (Centre)
    cv.strokeStyle = "#FF5555";
    cv.fillStyle = "#FF5555";
    cv.lineWidth = scale / 100;
    cv.beginPath();
    cv.moveTo(center.x + 8 * Math.cos(hRadian - a90deg), center.y + 8 * Math.sin(hRadian - a90deg));
    cv.lineTo(pos_x.x, pos_x.y);
    cv.lineTo(center.x + 8 * Math.cos(hRadian + a90deg), center.y + 8 * Math.sin(hRadian + a90deg));
    cv.closePath();
    cv.stroke();
    cv.fill();

    // メータの中心円を書く Ecrire le cercle central du compteur
    // グラデーション指定   Spécification de dégradé
    let grad2 = cv.createRadialGradient(center.x / 1, center.y / 1, 0, center.x, center.y, scale / 20);
    grad2.addColorStop(0.00, "#505050");
    grad2.addColorStop(0.95, "#515151");
    grad2.addColorStop(1.00, "#707070");
    cv.fillStyle = grad2;
    //cv.fillStyle = 'rgb(50, 50, 50)';
    cv.beginPath();
    cv.arc(center.x, center.y, scale / 20, 0, Math.PI * 2, false);
    cv.closePath();
    cv.fill();

    // 光沢を付ける（半透明の円描画） Gloss (dessin cercle translucide)
    cv.fillStyle = 'rgb(255, 255, 255, 0.07)';
    cv.beginPath();
    cv.arc(center.x, center.y / 20, scale / 1.5, 0, Math.PI * 2, false);
    cv.closePath();
    // cv.fill();           // probleme affichage sur tablette ACER
    //cv.restore();
}

// 針先端の座標取得 Coordonner l'acquisition d'une pointe d'aiguille
var cPoint = function (center, hookLength, radian) {
    return {
        x: center.x + hookLength * Math.cos(radian),
        y: center.y + hookLength * Math.sin(radian)
    }
};

function onClickCanvas(e) {
    if (PowerStatus == 0) {
        return;
    }

    e.preventDefault();

    var rect = e.target.getBoundingClientRect();
    var x = e.clientX - rect.left;
    var y = e.clientY - rect.top;
    var rx = x - CenterX;
    var ry = y - CenterY;
    CalcSpeedMeter(rx, ry);
}

function CalcSpeedMeter(rx, ry) {
    let aR = Math.sqrt(rx * rx + ry * ry);

    if ((aR < 40) || (aR > 200)) {
        /* 円の内側および外側は無視する Ignorer l'intérieur et l'extérieur du cercle */
        return;
    }

    var aTheta = Math.atan2(ry, rx) + Math.PI;
    var aTheta_r = (aTheta * 180 / Math.PI);

    /* 60deg 以上ずれている場合は無視する処理とする Ignorer le triangle équilateral ayant pour base l'image */
    if ((aTheta_r < 290) && (aTheta_r > 240)) {
        return;
    }

    /* 225deg - 315deg は無効範囲。n'est pas une plage valide */

    /* 許容範囲の調整 Réglage de la tolérance (0 speed) */
    if ((aTheta_r <= 320) && (aTheta_r >= 290)) {
        aTheta_r = 315;
    }

    /* 許容範囲の調整 Réglage de la tolérance(max speed) */
    if ((aTheta_r >= 220) && (aTheta_r <= 240)) {
        aTheta_r = 225;
    }

    /* 正規化 Normalisation */
    if (aTheta_r > 300) {
        aTheta_r = aTheta_r - 360 + 45;

    }
    else {
        aTheta_r = aTheta_r + 45;
    }

    //alert(aTheta_r);

    /* Conversion de l'angle à la valeur de la vitesse
    (en supprimant délibérément la précision pour réduire le poids du traitement 32b/64b) */
    LocSpeed = Math.round((aTheta_r) * maxStep / 270);

    document.getElementById('cran2traction').value = LocSpeed;
    //  document.getElementById('slider.slider').value = LocSpeed;
    //  $("#cran2traction").val(LocSpeed);
    $("#slider").slider("value", LocSpeed);
}

// Aiguillages et accessoires

function drawAccPanel() {
    var canvas = document.getElementById("accCanvas");
    var cv = canvas.getContext("2d");
    let adevie = document.getElementById("adevie");
    let adirect = document.getElementById("adirect");

    canvas.addEventListener("mousedown", onClickAccCanvas);
    canvas.addEventListener("touchstart", onClickAccCanvas);
    //document.getElementById("tab-acc").addEventListener("touchstart", onClickAccCanvas);
    //cv.save();

    // Dessin des carrés
    cvSize = canvas.getAttribute("width");
    cv.clearRect(0, 0, cvSize, cvSize);

    // Couleur de police
    cv.font = "16px 'arial'";

    // Affichage des aiguillages et des points (ronds)
    for (var y = 0; y < 4; y++) {
        for (var x = 0; x < 10; x++) {
            var xx = x * 90 + 1;
            var yy = y * 100 + 1;
            var index = (y + AccPageNo * 4) * 10 + x;

            cv.shadowBlur = 0;
            cv.fillStyle = '#EFEFEF';
            cv.strokeStyle = '#8F8F8F';
            cv.beginPath();
            cv.moveTo(xx, yy);
            cv.lineTo(xx + 80, yy);
            cv.lineTo(xx + 80, yy + 90);
            cv.lineTo(xx, yy + 90);
            cv.lineWidth = "1px";
            cv.lineCap = "round";
            cv.closePath();
            cv.fill();
            cv.stroke();

            if (index < 512) {
                cv.beginPath();
                if (index < nbreAiguillage) {
                    cv.drawImage((AccStatus[index] == 1) ? adevie : adirect, xx + 35, yy + 8);
                    cv.fillStyle = (AccStatus[index] == 0) ? "green" : "orange";
                }
                else {                 // Affichage des points ronds
                    cv.arc(xx + 40, yy + 55, 25, 0, Math.PI * 2, false);
                    cv.fillStyle = (AccStatus[index] == 0) ? "green" : "orange";  // direct,dévié
                }
                cv.fill();
                cv.fillText(index + 1, xx + 5, yy + 20);
            }
        }
    }
    //cv.restore();
};

function onClickAccPage(inPageCmd) {
    if (inPageCmd == 1) {
        AccPageNo++;
    }
    else {
        AccPageNo--;
    }

    if (AccPageNo < 0) {
        AccPageNo = 0;
    }

    if (AccPageNo > 12) {
        AccPageNo = 12;
    }

    drawAccPanel();
};

function onClickAccCanvas(e) {
    e.preventDefault();

    var rect = e.target.getBoundingClientRect();
    var x = e.clientX - rect.left;
    var y = e.clientY - rect.top;

    if(e.type == "touchstart") {
        x = e.touches[0].pageX - rect.left
        y = e.touches[0].pageY - rect.top
    }

    var xx = Math.floor(x / 90);
    var yy = Math.floor(y / 100);
    var index = (AccPageNo * 4 + yy) * 10 + xx;

    if (index < 512) {
        ChangeAcc(index);
        drawAccPanel();
    }
}

var isTouch = ('ontouchstart' in window);

$('#accCanvas').bind({
    'touchstart mousedown': function (e) {
        e.preventDefault();
        this.pageX = (isTouch ? event.changedTouches[0].pageX : e.pageX);
        this.pageY = (isTouch ? event.changedTouches[0].pageY : e.pageY);

        //alert(this.pageX + "," + this.pageY);
    },
    'touchmove mousemove': function (e) {
        e.preventDefault();
    },
    'touchend mouseup': function (e) {
    }
});

function onChangeNaiguillage() {
    nbreAiguillage = document.getElementById("nbreAiguillages").value;
    drawAccPanel();
}

// Retro-signalisation S88

function drawS88Panel() {
    var canvas = document.getElementById("s88Canvas");
    var cv = canvas.getContext("2d");

    // Dessin du carré
    cvSize = document.getElementById("s88Canvas").getAttribute("width");
    cv.clearRect(0, 0, cvSize, cvSize);
    // Canvas - Couleur de police
    cv.font = "16px 'arial'";

    // Affichage des rectangles
    for (var y = 0; y < 4; y++) {
        for (var x = 0; x < 16; x++) {
            var xx = x * 54 + 16;
            var yy = y * 100 + 1;
            var index = (y + S88PageNo * 4) * 16 + x;

            cv.shadowBlur = 0;
            cv.fillStyle = '#EFEFEF';
            cv.strokeStyle = '#8F8F8F';
            cv.beginPath();
            cv.moveTo(xx, yy);
            cv.lineTo(xx + 50, yy);
            cv.lineTo(xx + 50, yy + 80);
            cv.lineTo(xx, yy + 80);
            cv.lineWidth = "1px";
            cv.lineCap = "round";
            cv.closePath();
            cv.fill();
            cv.stroke();

            // Spécification de dégradé
            cv.shadowColor = '#9F9F9F';
            cv.shadowOffsetX = 0;
            cv.shadowOffsetY = 0;
            cv.shadowBlur = 8;

            // Affichage des points ronds
            if (S88Status[index] == 0) {        // libre
                cv.beginPath();
                cv.arc(xx + 25, yy + 50, 16, 0, Math.PI * 2, false);
                cv.fillStyle = (index < (s88Byte * 4) ? "green" : "lightgreen");
                cv.fill();
            }
            else if (S88Status[index] == 1) {   // occupé
                cv.beginPath();
                cv.arc(xx + 25, yy + 50, 16, 0, Math.PI * 2, false);
                cv.fillStyle = "red";
                cv.fill();
            }
            else {                              // indéfini
                cv.beginPath();
                cv.arc(xx + 25, yy + 50, 16, 0, Math.PI * 2, false);
                cv.fillStyle = "grey";
                cv.fill();
            }

            cv.fillStyle = "#A52A2A";
            cv.fillText(index + 1, xx + 18, yy + 20);
        }
    }
}

function onClickS88Page(inPageCmd) {
    if (inPageCmd == 1) {
        S88PageNo++;
    }
    else {
        S88PageNo--;
    }

    if (S88PageNo < 0) {
        S88PageNo = 0;
    }
    else if (S88PageNo > 7) {
        S88PageNo = 7;
    }

    drawS88Panel();
}
