// CDT31 by Philippe Chavatte - lormedy@free.fr - since 2018, july 25th
// Train controller written for DCCpp and S88 compatibility
// Use a PC serial link with "node.js" engine software, COMx ==> USB to Arduino MEGA with DCCpp software downloaded (@115200kbauds)
// Operates under control of 3 main files: CDT31.html, CDT31.js and CDT31server.js located in the same directory beside "node_modules" dir
// You can simultaneously control 99 locs and 512 accessories or turnouts on main and programming tracks
// Use 128 step to drive, speed counter is ajustable to your Loc maxi speed.
// Loc number use up to 5 numbers but only the last 4 are used as address.
// Display 2 x 256 sensors of the S88 retro-signalisation bus read thru DCCpp_S88 downloaded to Arduino MEGA
//
// Conduteur de train écrit pour être compatible avec DCCpp et S88 par Philippe Chavatte - lormedy@free.fr - 25 juillet 2018
// Utilise la liaison série du PC avec le moteur logiciel "node.js", COMx ==> USB vers un Arduino MEGA téléchargé avec le logiciel DCCpp (@115200kbauds)
// Fonctionne sous le controle de 3 fichiers principaux : CDT31.html, CDT31.js and CDT31server.js placés dans le répertoire qui contient aussi "node_modules"
// Vous pouvez controler simultanément 99 locomotives et 512 accessoires ou aiguillages, sur voie principale et de programmation
// Utilise 128 crans de traction, avec compteur de vitesse réglable à la vitesse maxi de votre loco,
// Numéro de loco sur 5 chiffres mais seulement les 4 derniers sont utilisés comme adresse.
// Affichage du bus de retro-signalisation S88 avec 2 x 256 capteurs lus par DCCpp_S88 téléchargé dans Arduino MEGA
//
// V1.0 commande DCCpp ou DCC++ - 10/09/2018
// V2.0 lecture et programmation des CV - 10/09/2018
// V2.1 lecture du courant sur voie principale - 15/12/2018
// V3.0 lecture du bus S88 compatible avec DCCpp_S88 - 15/04/2019
// V3.1 integration de websocket + json - 10/06/2019
//
// comm interface and websocket by Pierre N.
// software formating by Mathieu A.
// last update by Philippe Chavatte 2019, october 26th

const COMM_INTERFACE = 0;                        // Arduino USB/TxRx-Série = 0, Arduino Ethernet = 1
const urlDccpp = "http://192.168.0.200:2560/"    // IP de votre Arduino contenant le sketch DCC++, DCCpp ou DCCpp_S88
                                                 // DCCpp_S88 avec le shield WIZNET 5100 détecte automatiquement Ethernet
var fs = require('fs');
var http = require('http');
var url = require('url');
var path = require('Path')

/* -------------------------- Serial functions COM/USB --------------------- */

var usbserial = "COM3"; // default COM port
var Serialport = require('serialport');
const Readline = require('@serialport/parser-readline');
var serial_port = null;
var parser = null;
var dataReadBack = "";

// serial port auto detection
// returns promise that resolves to list of ports
Serialport.list()
.then(ports => {
    ports.forEach(com => {
        if (com.manufacturer.startsWith('Arduino') || com.manufacturer.startsWith('wch.cn')) {
            usbserial = com.comName;
            serial_port = new Serialport(usbserial, {
                baudRate: 115200,  // defaults for Arduino serial communication
                dataBits: 8,
                parity: 'none',
                stopBits: 1,
                flowControl: false
            });
            if (COMM_INTERFACE == 0) {
                parser = serial_port.pipe(new Readline({ delimiter: '\n' }))
                parser.on('data', readDCC);
            }
            console.log(`System Auto-connect serial port to ${usbserial}`);
        }
    });
});

/* -------------------------- COMM processing ------------------------------ */

function readDCC(data) { // read from Arduino serial/TCP port
    dataReadBack = data;
    console.log('DCC réponse : ' + data);
    if ( ( dataReadBack.startsWith("<y") ) || ( dataReadBack.startsWith("<p") ) || !( dataReadBack.startsWith("<") ) ) {
        if (wsClient != null) {
            var payload = {
                topic: "data",
                msg: data
            }
            wsClient.send(JSON.stringify(payload));            // send Arduino data to HTML GUI
        }
    }
}

function send2DCC(data) {
    let aRetryCounter = 0;
    let data2Arduino = "<" + data + ">"; // add DCCpp format

    //    console.log("send2DCC: " + data2Arduino.toString('hex') );
    if (COMM_INTERFACE == 0) {
        while (aRetryCounter < 10) {
            console.log('Cmd vers DCC: ' + data2Arduino.toString('hex'));

            if (writeData(data2Arduino) == true) {         // write thru Serial Port to Arduino
                return true;
            }

            setTimeout(5);
            aRetryCounter++;
        }
        return false;
    }
    else {
        wsMegaClient.send(data2Arduino);                   // write thru Ethernet Port to Arduino
        return true;
    }
}

function writeData(data) { // write to Arduino serial port
    if (serial_port === null) {
        console.log('Serial port is null !!!!!!');
        return false;
    }
    try {
        serial_port.write(data);
        return true;
    }
    catch (e) {
        console.log(e);
        return false;
    }
}

/* -------------------------- WebSocket Node to DCC ------------------------ */

const sleep = require('system-sleep');
var wait_answer = 0;
var wsClient = null;

var WebSocketServer = require('ws').Server;
var wss = new WebSocketServer({ port: 52560 });
wss.on('connection', function (ws) {
    wsClient = ws;
});

/*--------------------------- HTTP Serveur --------------------------------- */

http.createServer(function (request, response) {

    if( request.url.startsWith("/?") && request.method == 'GET' ) {
        var url_parts = url.parse(request.url, true);

        //Parse GET_Commands from GUI
        ParseCommand(url_parts.query.CMD);

        if (wait_answer > 0) sleep(wait_answer);    // par tranche de 1ms
        response.writeHead(200, { 'Content-Type': 'text/plain' });
        response.write(dataReadBack);
        response.end();
    }

    let filePath = '';
    if (request.url == '/') filePath = './views/CDT31.html';
    else filePath = `.${request.url}`;

    let extname = path.extname(filePath);
    let contentType = 'text/html';
    switch (extname) {
        case '.js':
            contentType = 'text/javascript';
            break;
        case '.css':
            contentType = 'text/css';
            break;
        case '.png':
            contentType = 'image/png';
            break;      
        case '.jpg':
            contentType = 'image/jpg';
            break;
        case '.ico':
            contentType = 'image/ico';
            break;
        case '.gif':
            contentType = 'image/gif';
            break;
    }

    fs.readFile(filePath, function(error, content) {
        if (error) {
            if(error.code == 'ENOENT'){
                response.writeHead(404);
                response.end('Not found');
                response.end(); 
            }
            else {
                response.writeHead(500);
                response.end(`Sorry, an error has occured: ${error.code} ..\n'`);
                response.end(); 
            }
        }
        else {
            response.writeHead(200, { 'Content-Type': contentType });
            response.end(content, 'utf-8');
        }
    });

}).listen(62560);

console.log('CDT31: Conducteur de Trains V3.1 - Philippe Chavatte 26/10/2019');
console.log('CDT31 running at http://127.0.0.1:62560/ on PC');
console.log('CDT31 running at http://your-IP:62560/ with Webphone or Tablet');

/* -------------------------- Command parser ------------------------------- */

// global variables
var Byte1 = 0;
var Byte2 = 0;
var Byte128 = 0;
var Byte160 = 0;
var Byte176 = 0;
var Byte222 = 0;
var Byte223 = 0;
var trackSelect = 0;
const PROG = 0;
const MAIN = 1;

function ParseCommand(inStrCommand) {
    if (inStrCommand == null) {
        return false;
    }

    let strCommand = inStrCommand;
    strCommand = strCommand.replace("(", ",");
    strCommand = strCommand.replace(")", ",");
    var strParameters = strCommand.split(",");

    // Vitesse, Direction
    if (strParameters[0] == "setLocoSpeedDir") {
        let aAddr = parseInt(strParameters[1]);
        let aSpeed = parseInt(strParameters[2]);
        let aDir = parseInt(strParameters[3]);

        aSpeed = (aSpeed > 127) ? 127 : aSpeed;     // aSpeed range 0..126
        let reg = (aAddr % 100) + 1;
        return send2DCC(`t${reg} ${aAddr} ${aSpeed} ${(aDir & 0x1)}`);
    }
    // Fonctions F0..F28
    else if (strParameters[0] == "setLocoFunction") {
        let aAddrFnc = parseInt(strParameters[1]);
        let aFuncNo = parseInt(strParameters[2]);
        let aFuncPw = parseInt(strParameters[3]);

        switch (aFuncNo) {
            case 0: (aFuncPw == 1) ? Byte128 |= 0x90 : Byte128 &= 0x8F; break;
            case 1: (aFuncPw == 1) ? Byte128 |= 0x81 : Byte128 &= 0x9E; break;
            case 2: (aFuncPw == 1) ? Byte128 |= 0x82 : Byte128 &= 0x9D; break;
            case 3: (aFuncPw == 1) ? Byte128 |= 0x84 : Byte128 &= 0x9B; break;
            case 4: (aFuncPw == 1) ? Byte128 |= 0x88 : Byte128 &= 0x97; break;
            case 5: (aFuncPw == 1) ? Byte176 |= 0xB1 : Byte176 &= 0xBE; break;
            case 6: (aFuncPw == 1) ? Byte176 |= 0xB2 : Byte176 &= 0xBD; break;
            case 7: (aFuncPw == 1) ? Byte176 |= 0xB4 : Byte176 &= 0xBB; break;
            case 8: (aFuncPw == 1) ? Byte176 |= 0xB8 : Byte176 &= 0xB7; break;
            case 9: (aFuncPw == 1) ? Byte160 |= 0xA1 : Byte160 &= 0xAE; break;
            case 10: (aFuncPw == 1) ? Byte160 |= 0xA2 : Byte160 &= 0xAD; break;
            case 11: (aFuncPw == 1) ? Byte160 |= 0xA4 : Byte160 &= 0xAB; break;
            case 12: (aFuncPw == 1) ? Byte160 |= 0xA8 : Byte160 &= 0xA7; break;
            case 13: (aFuncPw == 1) ? Byte222 |= 0x01 : Byte222 &= 0xFE; break;
            case 14: (aFuncPw == 1) ? Byte222 |= 0x02 : Byte222 &= 0xFD; break;
            case 15: (aFuncPw == 1) ? Byte222 |= 0x04 : Byte222 &= 0xFB; break;
            case 16: (aFuncPw == 1) ? Byte222 |= 0x08 : Byte222 &= 0xF7; break;
            case 17: (aFuncPw == 1) ? Byte222 |= 0x10 : Byte222 &= 0xEF; break;
            case 18: (aFuncPw == 1) ? Byte222 |= 0x20 : Byte222 &= 0xDF; break;
            case 19: (aFuncPw == 1) ? Byte222 |= 0x40 : Byte222 &= 0xBF; break;
            case 20: (aFuncPw == 1) ? Byte222 |= 0x80 : Byte222 &= 0x7F; break;
            case 21: (aFuncPw == 1) ? Byte223 |= 0x01 : Byte223 &= 0xFE; break;
            case 22: (aFuncPw == 1) ? Byte223 |= 0x02 : Byte223 &= 0xFD; break;
            case 23: (aFuncPw == 1) ? Byte223 |= 0x04 : Byte223 &= 0xFB; break;
            case 24: (aFuncPw == 1) ? Byte223 |= 0x08 : Byte223 &= 0xF7; break;
            case 25: (aFuncPw == 1) ? Byte223 |= 0x10 : Byte223 &= 0xEF; break;
            case 26: (aFuncPw == 1) ? Byte223 |= 0x20 : Byte223 &= 0xDF; break;
            case 27: (aFuncPw == 1) ? Byte223 |= 0x40 : Byte223 &= 0xBF; break;
            case 28: (aFuncPw == 1) ? Byte223 |= 0x80 : Byte223 &= 0x7F; break;
            default: break;
        }

        if (aFuncNo < 5) Byte1 = Byte128;           // selection des bytes à envoyer
        else if (aFuncNo < 9) Byte1 = Byte176;
        else if (aFuncNo < 13) Byte1 = Byte160;
        else if (aFuncNo < 21) {
            Byte1 = 222;
            Byte2 = Byte222;
        }
        else if (aFuncNo < 29) {
            Byte1 = 223;
            Byte2 = Byte223;
        }
        else Byte1 = 0;

        // 1er byte
        let param = `f ${aAddrFnc} ${Byte1}`;

        // 2eme byte, optionnel
        if ((Byte1 & 0xFE) == 222) {
            param += " ";
            param += Byte2;
        }

        return send2DCC(param);
    }
    // Aiguillages & accessoires
    else if (strParameters[0] == "setTurnout") { 
        let aAccAddr = parseInt(strParameters[1] & 0x7FF); // 0..2047
        let aAccSwitch = parseInt(strParameters[2]);

        let address_acc = ((aAccAddr / 4) & 0x1FF); // 0..511
        let sub_address_acc = aAccAddr % 4;         // 0..3
        let activate_Acc = aAccSwitch & 1;          // 0/1

        address_acc &= 0x7F; // 128 x 4 = 512, limite parmi 2044
        return send2DCC(`a ${address_acc} ${sub_address_acc} ${activate_Acc}`);
    }
    // Lire les CVs sur la voie de programmation
    else if (strParameters[0] == "getLocoConfig") {
        let trackSelect = parseInt(strParameters[2]);
        if (trackSelect == PROG) {                  // tracks : Prog = 0, Main = 1
            wait_answer = 2000;                     // delai to wait an answer from Arduino MEGA (read CV)
            let aCVNo = parseInt(strParameters[3]);
            send2DCC(`R ${aCVNo} 82 67`);
        }return
    }
    // Programmer les CVs sur la voie de programmation
    else if (strParameters[0] == "setLocoConfig") {
        let aAddr = parseInt(strParameters[1]);
        let trackSelect = parseInt(strParameters[2]);
        let aCVNo = parseInt(strParameters[3]);
        let aCVValue = parseInt(strParameters[4]);

        // voie de programmation
        if (trackSelect == PROG) {                  // tracks : Prog = 0, Main = 1
            return send2DCC(`W ${aCVNo} ${aCVValue} 87 67`);
        }
        // voie principale
        else {
            return send2DCC(`w ${aAddr} ${aCVNo} ${aCVValue}`);
        }
    }
    // Programmer les CVs sur la voie de programmation
    else if (strParameters[0] == "setLocoBitConfig") {
        let aAddr = parseInt(strParameters[1]);
        let trackSelect = parseInt(strParameters[2]);
        let aCVNo = parseInt(strParameters[3]);
        let aCVBitSelect = parseInt(strParameters[4]);
        let aCVBitValue = parseInt(strParameters[5]);

        // voie de programmation
        if (trackSelect == PROG) {                  // tracks : Prog = 0, Main = 1
            return send2DCC(`B ${aCVNo} ${aCVBitSelect} ${aCVBitValue} 87 98`);
        }
        // voie principale
        else {
            return send2DCC(`b ${aAddr} ${aCVNo} ${aCVBitSelect} ${aCVBitValue}`);
        }
    }
    // write DCC command
    else if (strParameters[0] == "sendCommand") {
        let command = strParameters[1];
        wait_answer = (command === "c" || command === "Y") ? 500 : 0;   // delay 0.5s

        // Formatage spécifique du message pour lire la retro-signalisation S88 avec DCCpp_S88 : Y N F
        // Nombre d'octets à lire : N = groupes de 8 capteurs (0..64) sur le bus S88 puis
        // Format du message : F = 0 ==> binAscii, F = 1 ==> NibbleAscii, F = 2 ==> Hexa, F = 3 ==> q/Q Sensor (0..511)
        if (command === "Y") {
            return send2DCC(`Y ${strParameters[2]} ${strParameters[3]}`);
        }
        else {
            return send2DCC(command);        // one char command
        }
    }
    else {
        return false;
    }
}
